"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isDevelopment = isDevelopment;
exports.toArray = toArray;
exports.toError = toError;
exports.validateFields = validateFields;
exports.findRelSchema = findRelSchema;
exports.findRelForm = findRelForm;
function isDevelopment() {
  return process.env.NODE_ENV !== "production";
}

function toArray(field) {
  if (Array.isArray(field)) {
    return field;
  } else {
    return [field];
  }
}

function toError(message) {
  if (isDevelopment()) {
    throw new ReferenceError(message);
  } else {
    console.error(message);
  }
}

function validateFields(action) {
  var toFields = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {
    return [];
  };

  if (!toFields) {
    toError("validateFields requires toFields function");
    return;
  }
  return function (params, schema) {
    var field = toFields(params);
    if (Array.isArray(field)) {
      field.filter(function (f) {
        return schema && schema.properties && schema.properties[f] === undefined;
      }).forEach(function (f) {
        return toError("Field  \"" + f + "\" is missing from schema on \"" + action + "\"");
      });
    } else if (field.indexOf(".") === -1 && schema && schema.properties && schema.properties[field] === undefined) {
      toError("Field  \"" + field + "\" is missing from schema on \"" + action + "\"");
    }
  };
}

function fetchSchema(ref, schema) {
  if (ref.startsWith("#/")) {
    ref.substr(2).split("/");
    return ref.substr(2).split("/").reduce(function (schema, field) {
      return schema[field];
    }, schema);
  } else {
    toError("Only local references supported at this point");
    return undefined;
  }
}

function toRefField(field, _ref) {
  var properties = _ref.properties;

  if (properties[field]) {
    if (properties[field]["$ref"]) {
      return properties[field]["$ref"];
    } else if (properties[field].items && properties[field].items["$ref"]) {
      return properties[field].items["$ref"];
    }
  }
  return undefined;
}

function findRelSchema(field, schema) {
  var separator = field.indexOf(".");
  if (separator === -1) {
    var ref = toRefField(field, schema);
    return ref ? fetchSchema(ref, schema) : schema;
  } else {
    var parentField = field.substr(0, separator);
    var _ref2 = toRefField(parentField, schema);
    if (_ref2) {
      var refSchema = fetchSchema(_ref2, schema);
      return findRelSchema(field.substr(separator + 1), refSchema);
    } else {
      return schema;
    }
  }
}

function findRelForm(field) {
  var formData = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var separator = field.indexOf(".");
  if (separator === -1) {
    return formData;
  } else {
    var parentField = field.substr(0, separator);
    return findRelForm(field.substr(separator + 1), formData[parentField]);
  }
}